/* *****************************************************************************
 * decode/mcs/program.c -- decode an MCS program.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libg1m.
 * libg1m is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libg1m is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libg1m; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libg1m/internals.h>

/**
 *	g1m_decode_mcs_program:
 *	Decode a program.
 *
 *	@arg	handle		the handle to make.
 *	@arg	buffer		the buffer to read from.
 *	@arg	head		the pre-filled head to complete and use.
 *	@return				the error code (0 if ok).
 */

int g1m_decode_mcs_program(g1m_mcsfile_t **handle, g1m_buffer_t *buffer,
	g1m_mcshead_t *head)
{
	int err;

	/* read header */
	DREAD(hd, mcs_programheader)
	log_info("Program password is '%.8s'.", hd.password);

	/* make final head and file */
	strncpy(head->password, (char*)hd.password, 8);
	head->password[8] = 0;
	head->size -= sizeof(struct mcs_programheader);
	head->size += 1; /* terminating zero - XXX: keep it? */
	err = g1m_make_mcsfile(handle, head);
	if (err) return (err);

	/* get content */
	g1m_mcsfile_t *h = *handle; size_t content_size = head->size - 1;
	log_info("Getting program content (%" PRIuSIZE "o)", content_size);
	GREAD(h->content, content_size);
	h->content[content_size] = 0;

	/* no error */
	return (0);
fail:
	g1m_free_mcsfile(*handle);
	*handle = NULL;
	return (err);
}
